/*
 * Copyright (c) 2023 Emma Tebibyte <emma@tebibyte.media>
 * SPDX-License-Identifier: AGPL-3.0-or-later
 *
 * This file is part of YAC coreutils.
 *
 * YAC coreutils is free software: you can redistribute it and/or modify it
 * under the terms of the GNU Affero General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or (at your
 * option) any later version.
 * 
 * YAC coreutils is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU Affero General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program. If not, see https://www.gnu.org/licenses/.
 */

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <sysexits.h>
#include <sys/stat.h>

/* Resolve a file from a path */
FILE *rpath(char *argv0, char *path) { 
	struct stat stats;
	FILE *file;

	if (path[0] == '-') {
		switch (path[1]) {
			case '\0':
				file = stdin;
				break;
			default:
				file = NULL;
				break;
		}
	} else if (stat(path, &stats) == 0 && S_ISDIR(stats.st_mode)) {
		fprintf(stderr, "%s: %s: Is a directory.\n", argv0, path);
		exit(EX_NOINPUT);
	} else if ((file = fopen(path, "r")) == NULL) {
		switch (errno) {
			case EACCES:
				fprintf(stderr, "%s: %s: Permission denied.\n", argv0, path);
				exit(EX_NOINPUT);
			case EISDIR:
				fprintf(stderr, "%s: %s: Is a directory.\n", argv0, path);
				exit(EX_NOINPUT);
			case ELOOP:
				fprintf(
					stderr,
					"%s: %s: Is a symbolic link loop.\n",
					argv0,
					path
				);
				exit(EX_UNAVAILABLE);
			case EMFILE:
				fprintf(stderr, "%s: Internal error.\n", argv0);
				exit(EX_SOFTWARE);
			case ENOENT: case ENOTDIR: case ENXIO:
				fprintf(
					stderr,
					"%s: %s: No such path or directory.\n",
					argv0,
					path
				);
				exit(EX_NOINPUT);
			default:
				fprintf(stderr, "%s: Unknown error.\n", argv0);
				exit(EX_UNAVAILABLE);
		}
	}
	return file;
}
